#!/usr/bin/python

import optparse
import os, errno
import crython
import socket, struct
import warnings

def main():

	usage = "usage: %prog [options] address:port dir"
	parser = optparse.OptionParser(usage)

	parser.add_option("-l", "--launch", dest="launch",
			help="launch streamer if needed")

	parser.add_option("-c", "--clobber", dest="clobber", action="store_true",
			help="clobber existing dumpdir")

	(options, args) = parser.parse_args()
	if len(args) != 2:
		parser.error("incorrect number of arguments")

	# Parse host and port
	ap = args[0].split(':')
	if len(ap) != 2:
		parser.error("couldn't parse address:port")
	(addr, port) = (ap[0], int(ap[1]))

	# Launch streamer if needed
	if options.launch is not None:
		ap = options.launch.split(':')
		cb_host = ap[0]
		if len(ap) == 2:
			cb_port = int(ap[1])
		elif len(ap) == 1:
			cb_port = crython.DEFAULT_CGI_PORT
		else:
			parser.error("unable to parse CryoBoard address")
			
		cb = crython.CryoBoard(cb_host, cb_port)
		streamer = crython.CryoStreamer(cb, addr, port)
		if not streamer.status():
			streamer.launch()
	else:
		streamer = crython.CryoStreamer(None, addr, port)

	# Parse directory
	dir = args[1]

	try: os.makedirs(dir)
	except OSError, e:
		if e.errno == errno.EEXIST:
			if not options.clobber:
				parser.error("Directory or file %s already exists!" % dir)
		elif e.errno == errno.EACCES:
			parser.error("Not allowed to create directory %s!" % dir)
		else:
			raise e

	first_pass = True
	format = open(os.path.join(dir, 'format'),'w')
	files = {}

	while True:

		# Grab samples
		x = streamer.get_samples(1)

		for drive_sense, d in x.iteritems():
			for i_q_rms, samples in d.iteritems():
				for chan in xrange(len(samples)):

					fn = '%s_%s_%02i' % (drive_sense, i_q_rms, chan)

					if first_pass:
						files[fn] = open(os.path.join(dir, fn), 'w')
						format.write(fn + '	RAW	S	1\n')

					files[fn].write(struct.pack('i', samples[chan][0]))
					files[fn].flush()

		if first_pass:
			format.close()
			first_pass = False


if __name__ == "__main__":
	main()

