#!/usr/bin/python

from fcgi import WSGIServer
import os, sys
import crython
import traceback
import urlparse

# Mako imports
import sys
sys.path.append('/usr/local/lib/python')
from mako.template import Template
from mako.lookup import TemplateLookup
from mako.exceptions import TopLevelLookupException

# JSON imports: use simplejson or Python 2.6 json, prefer simplejson.
try: import simplejson as json
except ImportError: import json

lookup = TemplateLookup(
	directories=['/home/www'],
	module_directory='/tmp/mako_modules',
	collection_size=100
)

cb = crython.CryoBoard('localhost')
s = crython.CryoStreamer(cb, 'localhost', crython.LOCAL_STREAMER_PORT)
try: s.launch()
except crython.CryoArgumentError: pass

backpack = {
	'crython': crython,
	'CryoBoard': cb,
	'CryoStreamer': s,
	'CryoConverter': cb.get_converter(),
	'CryoHeaterConverter': cb.get_heater_converter()
}

def crython_server(environ, start_response):

	# Retrieve and decode arguments
	try:
		args = environ['wsgi.input'].read()
		args = json.loads(args)
	except ValueError:
		args = {}

	try:	varargs = tuple(args['varargs'])
	except KeyError: varargs = []

	# Clean up kwargs (keys must be unicode)
	try:
		kwargs = args['kwargs']
		kwargs = dict([(k.encode('utf-8'), v) for k,v in kwargs.iteritems()])
	except KeyError: kwargs = {}

	try:	object = args['object']
	except KeyError: object = None

	# Determine if we're executing a template or a CGI-style call
	path = environ['SCRIPT_NAME'].split('/')

	# Read method from server
	method = path[-1].split('.')[0]

	if method == 'template':
		# Template calls: original URL available in REQUEST_URI
		decoded = urlparse.urlparse(environ['REQUEST_URI'])
		if decoded.path in ("/", ""):
			path = "index.tpl"
		else:
			path = decoded.path + ".tpl"

		try: template = lookup.get_template(path)
		except TopLevelLookupException:
			template = lookup.get_template("404.tpl")
			start_response('404 Not Found', [('Content-Type', 'text/html')])
			yield template.render(backpack=backpack, **kwargs)
		else:
			start_response('200 OK', [('Content-Type', 'text/html')])
			yield template.render(backpack=backpack, **kwargs)

	else:
		# CGI-style calls
		wrapped_result = {
			"server": {
				"pid": os.getpid()
			}
		}

		# Look up object
		try:	o = backpack[object]
		except Exception, e:
			object = None
			wrapped_result['error'] = {
				'where': 'backpack',
				'exception': str(type(e)),
				'args': e.args,
				'trace': traceback.format_exc()
			}

		# Look up method
		if object is not None:
			try:	method = getattr(o, method)
			except Exception, e:
				method = None
				wrapped_result['error'] = {
					'where': 'lookup',
					'exception': str(type(e)),
					'args': e.args,
					'trace': traceback.format_exc()
				}

		# Invoke method, catching exceptions
		if object is not None and method is not None:
			try:	wrapped_result['result'] = method(*varargs, **kwargs)
			except Exception, e:
				wrapped_result['error'] = {
					'where': 'invoke',
					'exception': str(type(e)),
					'args': e.args,
					'trace': traceback.format_exc()
				}

		# Return JSON-encoded response
		start_response('200 OK', [('Content-Type', 'text/plain')])
		yield json.dumps(wrapped_result)


WSGIServer(crython_server).run() 
